Package: RcppSimdJson
Type: Package
Title: 'Rcpp' Bindings for the 'simdjson' Header-Only Library for
        'JSON' Parsing
Version: 0.1.15
Date: 2026-01-14
Authors@R: c(person("Dirk", "Eddelbuettel", role = c("aut", "cre"), email = "edd@debian.org",
                    comment = c(ORCID = "0000-0001-6419-907X")),
             person("Brendan", "Knapp", role = "aut",
                    comment = c(ORCID = "0000-0003-3284-4972")),
             person("Daniel", "Lemire", role = "aut",
                    comment = c(ORCID = "0000-0003-3306-6922")))
Description: The 'JSON' format is ubiquitous for data interchange, and the
 'simdjson' library written by Daniel Lemire (and many contributors) provides a
 high-performance parser for these files which by relying on parallel 'SIMD'
 instruction manages to parse these files as faster than disk speed. See the
 <doi:10.48550/arXiv.1902.08318> paper for more details about 'simdjson'.  This
 package parses 'JSON' from string, file, or remote URLs under a variety of
 settings.
License: GPL (>= 2)
Imports: Rcpp, utils
LinkingTo: Rcpp
Suggests: bit64, tinytest
URL: https://github.com/eddelbuettel/rcppsimdjson/
BugReports: https://github.com/eddelbuettel/rcppsimdjson/issues
RoxygenNote: 7.1.1
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2026-01-14 12:10:02 UTC; edd
Author: Dirk Eddelbuettel [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-6419-907X>),
  Brendan Knapp [aut] (ORCID: <https://orcid.org/0000-0003-3284-4972>),
  Daniel Lemire [aut] (ORCID: <https://orcid.org/0000-0003-3306-6922>)
Maintainer: Dirk Eddelbuettel <edd@debian.org>
Repository: CRAN
Date/Publication: 2026-01-14 12:40:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 16:30:07 UTC; unix
Archs: RcppSimdJson.so.dSYM
